#include "stdafx.h"
#include <math.h>

#include "clf_object.h"
#include "clf_geometry.h"

static clf_object_manager* current_manager = NULL;

// object manager to dispatch commands from the application to the world objects
clf_object_manager::clf_object_manager()
{
	current_manager = this;
}

clf_object_manager::~clf_object_manager()
{
}

void clf_object_manager::add_object(clf_object* pob)
{
	push_back(pob);
}

void clf_object_manager::remove_object(clf_object* pob)
{
}

void clf_object_manager::frame_render()
{
	clf_object_manager::iterator it;
	for (it=begin(); it!=end(); ++it)
	{
		(*it)->render_object();
	}
}

void clf_object_manager::frame_move(float fElapsed)
{
	clf_object_manager::iterator it;
	for (it=begin(); it!=end(); ++it)
	{
		(*it)->frame_move(fElapsed);
	}
	// now is the right time to check for collisions
	for (it=begin(); it!=end(); ++it)
	{
		clf_object_manager::iterator next = it;
		++next;
		while (next != end())
		{
			if (polygons_collide((*it)->outline, (*next)->outline))
			{
				(*it)->stop();
				(*next)->stop();
			}
			++next;
		}
	}
}

void clf_object_manager::user_input(bool kdn[])
{
	clf_object_manager::iterator it;
	for (it=begin(); it!=end(); ++it)
	{
		(*it)->user_input(kdn);
	}
}

////////////////////////////////////////////////////////////////////


clf_car::clf_car()
{
	if (current_manager)
		current_manager->add_object(this);

	// position the car
	position[0] = 0.f;
	position[1] = 0.f;
	position[2] = 0.f;

	outline.push_back(clf_point2D(-.95f, -2.1f));
	outline.push_back(clf_point2D(-.95f, 2.1f));
	outline.push_back(clf_point2D(.95f, 2.1f));
	outline.push_back(clf_point2D(.95f, -2.1f));

	// setup initial speed and heading
	speed = 5.f;
	heading = 0.f;
	steering_angle = 0.f;
	wheelbase = 2.6f;
	acceleration = 0.f;
	moving = true;
}

clf_car::~clf_car()
{
}

void clf_car::render_object()
{
	if (!moving)
		glColor3f(1.0f,0.0f,0.0f);						// set the color to red
	else
		glColor3f(0.0f,1.0f,0.0f);						// set the color to green
	glBegin(GL_POLYGON);
		clf_polygon2D::iterator it;
		for (it=outline.begin(); it!=outline.end(); ++it)
		{
			glVertex3f( it->x, it->y, 0.0f);
		}
	glEnd();
}

void clf_car::frame_move(float fElapsed)
{
	if (!moving)
		return;
	// allow some dead angle around the center of the steering
	double turn = 0.f;
	if (fabs(steering_angle) > 0.5f)
	{
		double sa = steering_angle*3.1415926/180.0; // convert to radians
		double radius = wheelbase / sin(sa);        // calculate the turn radius
		double angular_v = speed / radius;          // calculate angular speed
		double dh = angular_v * fElapsed;           // calculate the anlge of turn for the elapsed time
		turn = dh;
		dh = dh*180.f/3.1415926;                    // convert back to degrees, (for OpenGL)
		heading += (float)dh;                       // and change the car direction
	}

	// convert degrees to radians for library functions cos, sin, YES this is overkill
	// math library (sin, cos, etc.) works with radians, but OpenGL expects degrees
	double ha = heading*3.1415926/180.0;
	// calculate distance covered
	float dist = (fElapsed * speed + 0.5f*acceleration*fElapsed*fElapsed);
	// calculate position
	float dx = dist * (float)sin(-ha);
	float dy = dist * (float)cos(ha);
	position[0] += dx;
	position[1] += dy;
	// calculate outline
	clf_polygon2D::iterator it;
	for (it=outline.begin(); it!=outline.end(); ++it)
	{
		(*it).x += dx;
		(*it).y += dy;
		if (fabs(steering_angle) > 0.5f)
			rotate2D(position[0], position[1], &(*it).x, &(*it).y, (float)turn);
	}
	// speed change = acceleration*elapsedTime
	speed += acceleration * fElapsed;
}

void clf_car::stop()
{
	moving = false;
}

void clf_car::user_input(bool keyDown[])
{
	acceleration = 0.f;
	if (keyDown['A']) // faster
	{
		acceleration = 10.f;
	}
	if (keyDown['Z']) // slower
	{
		acceleration = -10.f;
	}
	if (keyDown['J']) // bring steering to straight
		steering_angle = 0.f;
	if (keyDown['N']) // left
	{
		steering_angle += 1.f;
		if (steering_angle > 40.f)  // maximum steering angle
			steering_angle = 40.f;
	}
	if (keyDown['M']) // right
	{
		steering_angle -= 1.f;
		if (steering_angle < -40.f)
			steering_angle = -40.f;
	}
	if (keyDown['R']) // reset
	{
		// position the car
		position[0] = 0.f;
		position[1] = 0.f;
		position[2] = -80.f;
		// setup initial speed and heading
		speed = 5.f;
		heading = 0.f;
		steering_angle = 0.f;
		outline.clear();
		outline.push_back(clf_point2D(-.95f, -2.1f));
		outline.push_back(clf_point2D(-.95f, 2.1f));
		outline.push_back(clf_point2D(.95f, 2.1f));
		outline.push_back(clf_point2D(.95f, -2.1f));
		moving=true;
	}
}

///////////////////////////////////////////////////////////////////////////////

clf_building::clf_building()
{
	if (current_manager)
		current_manager->add_object(this);
}

clf_building::~clf_building()
{
}

void clf_building::render_object()
{
	glColor3f(.8f,.8f,.8f);						// set the color
	glBegin(GL_POLYGON);
		clf_polygon2D::iterator it;
		for (it=outline.begin(); it!=outline.end(); ++it)
		{
			glVertex3f( it->x, it->y, 0.0f);
		}
	glEnd();
}

void clf_building::setup(clf_polygon2D& _ol)
{
	outline.clear();		
	clf_polygon2D::iterator it;
	for (it=_ol.begin(); it!=_ol.end(); ++it)
	{
		outline.push_back(*it);
	}
}
